<?php declare(strict_types=1);

namespace App\Service;

use App\Entity\Printer;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Serializer\SerializerInterface;

class PrinterService
{
    private const REQUIRED_PROPERTIES = ['name', 'price', 'build_volume', 'max_speed'];

    public function __construct(private readonly EntityManagerInterface $entityManager, private readonly SerializerInterface $serializer)
    {
    }

    public function deletePrinter(Printer $printer): void
    {
        $this->entityManager->remove($printer);
        $this->entityManager->flush();
    }

    public function createPrinter(string $jsonString): Printer
    {
        $printer = $this->serializer->deserialize($jsonString, Printer::class, 'json');
        $this->entityManager->persist($printer);
        $this->entityManager->flush();

        return $printer;
    }

    public function validateJson(string $jsonString): bool
    {
        $array = json_decode($jsonString, true);
        return array_keys($array) == self::REQUIRED_PROPERTIES;
    }

    public function editPrinter(Printer $printer, string $json): Printer
    {
        $jsonArray = json_decode($json, true);

        $printer->setName($jsonArray['name'] ?? $printer->getName());
        $printer->setPrice($jsonArray['price'] ?? $printer->getPrice());
        $printer->setBuildVolume($jsonArray['build_volume'] ?? $printer->getBuildVolume());
        $printer->setMaxSpeed($jsonArray['max_speed'] ?? $printer->getMaxSpeed());

        $this->entityManager->flush();

        return $printer;
    }
}